package com.streambase.contrib.chartdemo;

import com.streambase.sb.Tuple;
import com.streambase.sb.client.DequeueResult;
import com.streambase.sb.client.StreamBaseClient;

/**
 * An example of reusing the same charting code that executes in-process,
 * but this time by sending values through as a result of dequeuing
 * tuples from StreamBase using the StreamBase Client API (as opposed
 * to calling the static method {@link GraphMVC#chartStockPrice(com.streambase.sb.Timestamp, double)}
 * directly from sbd in a Map operator).
 * <p/>
 * To use, ensure the GraphingExample.sbapp application file has the
 * module parameter Stock
 */
public class GraphClient {
	public static void main(String[] args) throws Exception {
		// Note to the reader: this is an incredibly simplified
		// way of connecting to StreamBase (to the default URI)
		// and dequeuing immediately in the main thread and pushing
		// data to, in this case, the charting routine.
		// This is most certainly *not* best practice -- typically,
		// you should use a threaded dequeuer that dequeues data
		// as fast as it can, and you should then use concurrent
		// collections to pop off results into your application logic.
		//
		// Using the StreamBase Studio Java client wizards is a good start
		// when writing real clients.
		StreamBaseClient client = new StreamBaseClient();
		client.subscribe("Feed_Out");
		DequeueResult dr;
		while ((dr = client.dequeue()) != null) {
			if (dr.getStatus() == DequeueResult.GOOD) {
				for (Tuple tuple : dr) {
					GraphMVC.chartStockPrice(
							tuple.getTimestamp("Date"),
							tuple.getDouble("AdjClose"));
				}
			}
		}
		
	}
}
