package com.streambase.contrib.chartdemo;

import java.util.Observer;

import com.streambase.contrib.chartdemo.controllers.MessageController;
import com.streambase.contrib.chartdemo.models.DataModel;
import com.streambase.contrib.chartdemo.views.LogView;
import com.streambase.contrib.chartdemo.views.StockPriceChartView;
import com.streambase.sb.StreamBaseException;
import com.streambase.sb.Timestamp;

public class GraphMVC {

	private static DataModel          m_Model;
	private static MessageController  m_Control;
	private static boolean            m_Init = false;

	private static void initIfNeeded()
	{
		if( false == m_Init )
		{
			m_Model = new DataModel();
			m_Control = new MessageController( m_Model );
			m_Init = true;
			m_Control.SetPort( 1026 );
		}
	}

	public static double chartStockPrice( Timestamp time, double inStockValue )
	{
		initIfNeeded();

		String mesg = time + ";" + Double.toString( inStockValue );
		if( false == m_Control.IsReceiverRunning() )
		{
			LogView lv = new LogView();
			StockPriceChartView spcv = new StockPriceChartView( "MainChartView", m_Control, m_Model );

			Observer[] views = { lv, spcv };
			m_Model.AddViewers( views );

			spcv.pack();
			spcv.setVisible( true );

			new Thread(m_Control.StartReceiver).start();
		}
		else
			m_Control.SendMessageString( mesg );
		return inStockValue;
	}

	public static void main( String[] args )
	{
		if ( 2 != args.length ) 
		{
			System.err.println("\nUSAGE: [Timestamp String] [Stock value]");
			System.exit( -1 );
		}

		initIfNeeded();
		Timestamp time = null;
		try {
			time = Timestamp.fromString( args[0].toString() );
		} catch( StreamBaseException e ) {
			System.err.println("::GraphingExampleDemo:: error converting string " + args[0] + " to timestamp");
			e.printStackTrace();
			System.exit( -1 );
		}
		double InStockValue = Double.valueOf(args[1]).doubleValue();
		System.out.println("::GraphingExampleDemo:: Sending chart " + time + " " + InStockValue);
		chartStockPrice( time, InStockValue );
	} 

}
