package com.streambase.contrib.chartdemo.controllers;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.Socket;
//import java.net.UnknownHostException;
import java.util.Date;

import com.streambase.contrib.chartdemo.models.DataModel;
import com.streambase.sb.StreamBaseException;
import com.streambase.sb.Timestamp;

// has logic for two threads:
// Message Receiver, Message Sender
public class MessageController {

	private int        m_Port;
	private DataModel  m_Model;
	private boolean    m_Exit;

	public void SetPort( int port ) { m_Port = port; }
	public void ExitNow()           { m_Exit = true; }
	public boolean IsDone()         { return m_Exit; }
	
	public void SendMessageString( String message )
	{
		Socket clientSocket = null;
		PrintWriter outstream = null;
		boolean ready = true;
// Open connection
		try	{
			clientSocket = new Socket( InetAddress.getLocalHost(), m_Port );
			outstream = new PrintWriter( clientSocket.getOutputStream(), true );
		} catch( IOException e ) {
			System.err.println("::Controller:: Sender unable to create connection to port " + m_Port);
			e.printStackTrace();
			ready = false;
		}
// Send message
		if( ready ) outstream.println( message.toString() );
// Close connection
		try {
			outstream.close();
			clientSocket.close();
		} catch( IOException e ) {
			System.err.println("::Controller:: Sender unable to close connection");
			e.printStackTrace();
		}
	}

	public boolean IsReceiverRunning()
	{
		Socket clientSocket = null;

		// if client can't connect, receiver not running
		try {
			clientSocket = new Socket( InetAddress.getLocalHost(), m_Port );
			clientSocket.setSoTimeout(1000);
		} catch( IOException e ) {
			return false; 
		}
		
		try {
			clientSocket.close();
		} catch( IOException e ) {
			System.err.println("::Controller:: Error closing socket");
			e.printStackTrace();
		}
		return true;
	}

	public boolean isPortOpen()
	{
		Socket clientSocket = null;
		// if client can't connect, receiver not running
		try {
			clientSocket = new Socket( InetAddress.getLocalHost(), m_Port );
		} catch( IOException e ) { return false; }

		try {
			clientSocket.close();
		} catch( IOException e ) {
			System.err.println("::Controller:: Error closing socket");
			e.printStackTrace();
		}
		return true;
	}


	public Runnable StartReceiver = new Runnable()
	{
		public void run()
		{
			ServerSocket listenSocket = null;
			try {
				listenSocket = new ServerSocket( m_Port, 5, InetAddress.getLocalHost() );				
			} catch( IOException e ) {
				System.err.println("::Controller:: Receiver could not listen on port " + m_Port);
				e.printStackTrace();
				System.exit( -1 );
			}
			
			while( true )
			{
				if( m_Exit ) break;
				Socket sock = null;
				BufferedReader instream = null;
				Double stockPrice = null;
				Date timestamp = null;
// Accept connection
				try {
					sock = listenSocket.accept();
				} catch( IOException e ) {
					System.err.println("::Controller:: Receiver failed to accept connection on port " + m_Port);
					e.printStackTrace();
				}
// Read stock price from connection
				try {
					instream = new BufferedReader( new InputStreamReader(sock.getInputStream()) );
					String mesg = instream.readLine();
					if( null == mesg ) continue; // null mesg comes from IsReceiverRunning test
					String tokens[] = mesg.split(";");
					timestamp = Timestamp.fromString(tokens[0]).toDate();
					stockPrice = Double.valueOf( tokens[1] );
				} catch( IOException e ) {
					System.err.println("::Controller:: Receiver had error reading message");
					e.printStackTrace();
				} catch( StreamBaseException e ) {
					System.err.println("::Controller:: Receiver could not convert timestamp string to Date");
					e.printStackTrace();
				}  catch( NumberFormatException e ) {
					System.err.println("::Controller:: Receiver could not convert stock price to double");
					e.printStackTrace();
				}
// Add stock price to data model
				m_Model.addValueToTimeSeries( timestamp, stockPrice.doubleValue() );
// Close connection
				try {
					sock.close();
				} catch( IOException e ) {
					System.err.println("::Controller:: Receiver could not close socket");
					e.printStackTrace();
				}
			}
			try {
				listenSocket.close();
				System.out.println("::Controller:: port " + m_Port + " closed");
			} catch( IOException e ) {
				System.err.println("::Controller:: error closing port " + m_Port);
				e.printStackTrace();
			}
		}
	};

	public MessageController( DataModel dm )
	{
		super();
		m_Model = dm;
		m_Exit = false;
	}

}
