package com.streambase.contrib.chartdemo.views;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.Dimension;

import javax.swing.JButton;
import javax.swing.JPanel;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.chart.plot.XYPlot;
import org.jfree.data.time.TimeSeriesCollection;
import org.jfree.data.xy.XYDataset;
import org.jfree.ui.ApplicationFrame;

import com.streambase.contrib.chartdemo.controllers.MessageController;
import com.streambase.contrib.chartdemo.models.DataModel;

import java.util.Observable;
import java.util.Observer;


public class StockPriceChartView extends ApplicationFrame implements ActionListener, Observer {
	
	public DataModel m_Model;
	private MessageController m_Controller;
	
	private static final long serialVersionUID = 1L;

	public void update( Observable thing, Object obj ) { }	

	private TimeSeriesCollection createDataset()
	{
		TimeSeriesCollection dataset = new TimeSeriesCollection( m_Model.StockPrices );
		return dataset;
	}
	
	private JFreeChart createChart( final XYDataset dataset )
	{
		final JFreeChart result = ChartFactory.createTimeSeriesChart(
				// title, timeAxisLabel, valueAxisLabel
				"Stock Price Chart", "Time", "Stock Price",
				// dataset, legend, tooltips, URLs
				dataset, false, true, false
		);
		final XYPlot plot = result.getXYPlot();
		ValueAxis axis = plot.getDomainAxis();
		axis.setAutoRange( true );
		axis = plot.getRangeAxis();
		axis.setAutoRange( true );
		return result;
	}

	public StockPriceChartView( String title, MessageController c, DataModel dm )
	{
		super( title );
		this.setDefaultCloseOperation( ApplicationFrame.EXIT_ON_CLOSE );
		
		m_Model = dm;
		m_Controller = c;
	
		final TimeSeriesCollection dataset = createDataset();
		final JFreeChart chart = createChart( dataset );
		ChartPanel chartPanel = new ChartPanel( chart, false );
		chartPanel.setPreferredSize( new Dimension(500,270) );
		this.add( chartPanel, BorderLayout.CENTER );
		
		final JButton plotButton = new JButton( "Plot Histogram" );
		plotButton.setActionCommand( "PLOT_HISTOGRAM" );
		plotButton.addActionListener( this );
	
		final JPanel content = new JPanel();
		this.add( content, BorderLayout.NORTH );
		content.add( plotButton );
	}

	public void actionPerformed( ActionEvent e )
	{
		if( e.getActionCommand().equals("PLOT_HISTOGRAM") )
		{
			if( 0 != m_Model.getSeriesCount() )
			{
				StockReturnsHistogramView StockReturnsView = new StockReturnsHistogramView( "ReturnChartView", m_Model );
				StockReturnsView.showChart();
			}
		}
	}
	
	public void windowClosing( final WindowEvent evt )
	{
		if( this == evt.getWindow() )
		{
			m_Controller.ExitNow();
			// wait for controller to be done
			do {}
			while( false == m_Controller.IsDone() );
			dispose();
			System.exit( NORMAL );
		}
	}

}
