package com.streambase.contrib.chartdemo.views;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.WindowEvent;
import java.util.Random;

import javax.swing.JPanel;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.data.statistics.HistogramDataset;
import org.jfree.data.statistics.HistogramType;
import org.jfree.data.xy.IntervalXYDataset;
import org.jfree.ui.ApplicationFrame;

import com.streambase.contrib.chartdemo.models.DataModel;

public class StockReturnsHistogramView extends ApplicationFrame {

	private static final long serialVersionUID = 1L;
	private DataModel m_Model;

	private IntervalXYDataset createDataset()
	{
		Random r = new Random(); // for sampling from Gaussian distribution
		HistogramDataset dataset = new HistogramDataset();
		dataset.setType( HistogramType.FREQUENCY );
		int count = m_Model.getSeriesCount();
		double[] actualValues = new double[count];
		double[] gaussModelValues = new double[count];
		for( int i=0; i < count-1; i++ )
		{
			// get difference (return) for two consecutive stock prices
			actualValues[i] = m_Model.getItemCount(i+1).doubleValue() - m_Model.getItemCount(i).doubleValue();
			// simulate the return from a Gaussian distribution
			gaussModelValues[i] = r.nextGaussian();
		}
		dataset.addSeries( "Actual Returns", actualValues, 60 ); 
		dataset.addSeries( "Gaussian Model Returns", gaussModelValues, 30 ); 
		return (IntervalXYDataset) dataset;
	}

	private JFreeChart createChart( final IntervalXYDataset dataset )
	{
		final JFreeChart result  = ChartFactory.createHistogram(
				 // title, domainAxisLabel, rangeAxisLabel
				"Daily Stock Returns Histogram", "number", "value",
				// dataset, orientation, legend, tooltips, URLs
				dataset, PlotOrientation.VERTICAL, true, true, false 
		);

		result.getXYPlot().setForegroundAlpha( 0.5f );
		final XYPlot plot = result.getXYPlot();
		ValueAxis axis = plot.getDomainAxis();
		// set the Domain-Range axes of the X-Y scatter plot
		axis.setAutoRange( true );
		axis = plot.getRangeAxis();
		axis.setAutoRange( true );
		return result;
	}

	public StockReturnsHistogramView( String title, DataModel dm )
	{
		super( title );
		m_Model = dm;

		final IntervalXYDataset dataset = createDataset();       
		final JFreeChart chart = createChart( dataset );
		final ChartPanel chartPanel = new ChartPanel( chart, false );
		chartPanel.setPreferredSize( new Dimension(500, 270) );

		final JPanel content = new JPanel( new BorderLayout() );
		content.add( chartPanel );
		setContentPane( content );
	}   

	public void showChart()
	{
		this.pack();
		this.setVisible( true );
	}

	public void windowClosing( final WindowEvent evt )
	{
		if( this == evt.getWindow() )
		{
			dispose();
		}
	}

}
